/*
 * Copyright 2004-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.hdiv.webflow.listener;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hdiv.dataComposer.IDataComposer;
import org.hdiv.util.HDIVUtil;
import org.springframework.webflow.context.servlet.ServletExternalContext;
import org.springframework.webflow.core.collection.AttributeMap;
import org.springframework.webflow.execution.FlowExecutionListenerAdapter;
import org.springframework.webflow.execution.FlowSession;
import org.springframework.webflow.execution.RequestContext;
import org.springframework.webflow.execution.repository.BadlyFormattedFlowExecutionKeyException;

/**
 * HDIV listener for webflow to delete HDIV pages from session in order to optimize memory consumption.
 * 
 * @author Gorka Vicente
 * @since 2.0.3
 */
public class HDIVFlowExecutionListener extends FlowExecutionListenerAdapter {

	private static Log log = LogFactory.getLog(HDIVFlowExecutionListener.class);
	
	/**
	 * The default conversation id prefix delimiter ("_c").
	 */
	private static final String CONVERSATION_ID_PREFIX = "_c";

	/**
	 * The default continuation id prefix delimiter ("_k").
	 */
	private static final String CONTINUATION_ID_PREFIX = "_k";

	/**
	 * The format of the default string-encoded form, as returned by toString().
	 */
	private static final String FORMAT = CONVERSATION_ID_PREFIX + "<conversationId>" + CONTINUATION_ID_PREFIX
			+ "<continuationId>";

	
	/**
	 * Called when a client request has completed processing.
	 * 
	 * @param context the source of the event
	 */
	public void requestProcessed(RequestContext context) {			
		this.addFlowId(context);		
	}	
	
	/**
	 * Adds the flow identifier to the page of type <code>IPage</code>.
	 * 
	 * @param request HTTP servlet request
	 * @since HDIV 2.0.3
	 */		
	public void addFlowId(RequestContext context) {
		
		HttpServletRequest request = ((ServletExternalContext) context.getExternalContext()).getRequest();		
		IDataComposer dataComposer = HDIVUtil.getDataComposer(request);
		
		String _flowExecutionValue = context.getExternalContext().getRequestParameterMap().get("_flowExecutionKey");
		
		if (_flowExecutionValue != null) {
			dataComposer.addFlowId(_flowExecutionValue);
		}
	}

	/**
	 * Called when a flow execution session ends. If the ended session was the
	 * root session of the flow execution, the entire flow execution also ends.
	 * 
	 * @param context the source of the event
	 * @param session ending flow session
	 * @param output final, unmodifiable output returned by the ended session
	 */
	public void sessionEnded(RequestContext context, FlowSession session, AttributeMap output) {
		
		try {
			if (!context.getFlowExecutionContext().isActive()) {
				
				String encodedKey = context.getExternalContext().getRequestParameterMap().get("_flowExecutionKey");
				String[] keyParts = this.keyParts(encodedKey);
				
				HDIVUtil.getSession().removeEndedPages(keyParts[0]);
				if (log.isDebugEnabled()) {
					log.debug("Flow ended:" + keyParts[0]);
				}
			}

		} catch (BadlyFormattedFlowExecutionKeyException e) {			
			log.error("The string encoded flow execution key is invalid");
			throw e;
		}
	}
	
	/**
	 * Helper that splits the string-form of an instance of this class into its
	 * "parts" so the parts can be easily parsed.
	 * 
	 * @param encodedKey the string-encoded composite flow execution key
	 * @return the composite key parts as a String array (conversationId = 0,
	 *         continuationId = 1)
	 */
	private String[] keyParts(String encodedKey) throws BadlyFormattedFlowExecutionKeyException {

		if (!encodedKey.startsWith(CONVERSATION_ID_PREFIX)) {
			throw new BadlyFormattedFlowExecutionKeyException(encodedKey, FORMAT);
		}
		int continuationStart = encodedKey.indexOf(CONTINUATION_ID_PREFIX, CONVERSATION_ID_PREFIX.length());
		if (continuationStart == -1) {
			throw new BadlyFormattedFlowExecutionKeyException(encodedKey, FORMAT);
		}
		String conversationId = encodedKey.substring(0, continuationStart);
		String continuationId = encodedKey.substring(continuationStart);

		return new String[] { conversationId, continuationId };
	}

}
